@@ -64,3 +64,8 @@ def import_translations_from(archive):
 if __name__ == '__main__':
 	assert len(sys.argv) == 2 and sys.argv[1].endswith('.tar.gz')
 	import_translations_from(sys.argv[1])
+	
+	import sys
+	sys.path.insert(0, '.')
+	from tests.translations import TestTranslations
+	TestTranslations().runTest(verbose=True)
