@@ -1,13 +1,19 @@
-# TODO: Licence Header and Copyright
+# -*- coding: utf-8 -*-
+
+# Copyright 2008 Johannes Reinhardt <jreinhardt@ist-dein-freund.de>
 
 '''This modules handles export of LaTeX Code'''
 
 import re
+import logging
 
 from zim.fs import File
 from zim.formats import *
 from zim.parsing import TextBuffer
 
+logger = logging.getLogger('zim.formats.latex')
+
+
 info = {
 	'name':		'LaTeX',
 	'mime': 	'application/x-tex',
@@ -81,9 +87,10 @@ class Dumper(DumperClass):
 		self.document_type = self.template_options.get('document_type')
 			# Option set in template - potentially tainted value
 		if not self.document_type in ('report', 'article','book'):
+			logger.warn('No document type set in template, assuming "report"')
 			self.document_type = 'report' # arbitrary default
-
-		logger.info('used document type: %s'%self.document_type)
+		else:
+			logger.info('used document type: %s'%self.document_type)
 
 		output = TextBuffer()
 		self.dump_children(tree.getroot(), output)
@@ -135,6 +142,10 @@ class Dumper(DumperClass):
 				output.append('\n\\begin{verbatim}\n')
 				output.extend(myoutput)
 				output.append('\n\\end{verbatim}\n')
+			elif element.tag == 'sub':
+				output.append('$_{%s}$' % element.text)
+			elif element.tag == 'sup':
+				output.append('$^{%s}$' % element.text)
 			elif element.tag == 'img':
 				#we try to get images about the same visual size, therefore need to specify dot density
 				#96 dpi seems to be common for computer monitors
@@ -146,9 +157,8 @@ class Dumper(DumperClass):
 						equri = self.linker.link(element.attrib['src'])
 						eqfid = File(url_decode(equri[:-4] + '.tex'))
 						output.append('\\begin{math}\n')
-						output.extend(eqfid.readlines())
+						output.extend(eqfid.read().strip())
 						output.append('\n\\end{math}')
-						eqfid.close()
 					except:
 						logger.exception('Could not find latex equation:')
 					else:
