@@ -290,6 +290,14 @@ class OverWriteError(Error):
 	pass # TODO description
 
 
+class FileNotFoundError(PathLookupError):
+
+	def __init__(self, file):
+		self.file = file
+		self.msg = _('No such file: %s') % file.path
+			# T: message for FileNotFoundError
+
+
 # TODO actually hook the signal for deleting files and folders
 
 class _FS(gobject.GObject):
@@ -524,7 +532,7 @@ class WindowsPath(UnixPath):
 	def uri(self):
 		'''File uri property with win32 logic'''
 		# win32 paths do not start with '/', so add another one
-		return 'file:///'+self.canonpath
+		return 'file:///' + url_encode(self.canonpath)
 
 	@property
 	def canonpath(self):
@@ -630,8 +638,8 @@ class Dir(Path):
 		'''
 		assert self.path and self.path != '/'
 		logger.info('Remove file tree: %s', self)
-		for root, dirs, files in os.walk(self.encodedpath, topdown=False, followlinks=False):
-			# walk should not decent into symlinked folders
+		for root, dirs, files in os.walk(self.encodedpath, topdown=False):
+			# walk should not decent into symlinked folders by default
 			# remove() and rmdir() both should remove a symlink rather
 			# than the target of the link
 			for name in files:
@@ -763,11 +771,10 @@ class File(Path):
 		'''Returns True if the file exists and is actually a file'''
 		return os.path.isfile(self.encodedpath)
 
-	def open(self, mode='r', encoding='utf-8'):
+	def open(self, mode='r'):
 		'''Returns an io object for reading or writing.
 		Opening a non-exisiting file for writing will cause the whole path
 		to this file to be created on the fly.
-		To open the raw file specify 'encoding=None'.
 		'''
 		assert mode in ('r', 'w')
 		if mode == 'w':
@@ -778,24 +785,19 @@ class File(Path):
 			else:
 				pass # exists and writable
 
-		if encoding:
-			mode += 'b'
-
-		if mode in ('w', 'wb'):
+		mode += 'b'
+		if mode == 'wb':
 			tmp = self.encodedpath + '.zim.new~'
 			fh = FileHandle(tmp, mode=mode, on_close=self._on_write)
 		else:
 			fh = open(self.encodedpath, mode=mode)
 
-		if encoding:
-			# code copied from codecs.open() to wrap our FileHandle objects
-			info = codecs.lookup(encoding)
-			srw = codecs.StreamReaderWriter(
-				fh, info.streamreader, info.streamwriter, 'strict')
-			srw.encoding = encoding
-			return srw
-		else:
-			return fh
+		# code copied from codecs.open() to wrap our FileHandle objects
+		info = codecs.lookup('utf-8')
+		srw = codecs.StreamReaderWriter(
+			fh, info.streamreader, info.streamwriter, 'strict')
+		srw.encoding = 'utf-8'
+		return srw
 
 	def _on_write(self):
 		# flush and sync are already done before close()
@@ -825,12 +827,27 @@ class File(Path):
 		if isnew:
 			FS.emit('path-created', self)
 
-	def read(self, encoding='utf-8'):
-		'''Returns the content as string, or an empty string if
-		this file does not exist.
+	def raw(self):
+		'''Like read() but without encoding and newline logic.
+		Used to read binary data, e.g. when serving files over www.
+		Note that this function also does not integrates with checking
+		mtime, so intended for read only usage.
 		'''
 		with self._lock:
-			text = self._read(encoding)
+			try:
+				fh = open(self.encodedpath, mode='rb')
+				content = fh.read()
+				fh.close()
+				return content
+			except IOError:
+				raise FileNotFoundError(self)
+
+	def read(self):
+		'''Returns the content as string. Raises a
+		FileNotFoundError exception when the file does not exist.
+		'''
+		with self._lock:
+			text = self._read()
 		return text
 
 	def read_async(self, callback=None, data=None):
@@ -838,23 +855,25 @@ class File(Path):
 		Returns a AsyncOperation object, see there for documentation
 		for 'callback'. Try operation.result for content.
 		'''
+		if not self.exists():
+			raise FileNotFoundError(self)
 		operation = AsyncOperation(
 			self._read, lock=self._lock, callback=callback, data=data)
 		operation.start()
 		return operation
 
-	def _read(self, encoding='utf-8'):
-		if not self.exists():
-			return ''
-		else:
-			file = self.open('r', encoding)
+	def _read(self):
+		try:
+			file = self.open('r')
 			content = file.read()
 			self._checkoverwrite(content)
 			return _convert_newlines(content)
+		except IOError:
+			raise FileNotFoundError(self)
 
 	def readlines(self):
-		'''Returns the content as list of lines, or an empty list if
-		this file does not exist.
+		'''Returns the content as list of lines. Raises a
+		FileNotFoundError exception when the file does not exist.
 		'''
 		with self._lock:
 			lines = self._readlines()
@@ -865,19 +884,21 @@ class File(Path):
 		Returns a AsyncOperation object, see there for documentation
 		for 'callback'. Try operation.result for content.
 		'''
+		if not self.exists():
+			raise FileNotFoundError(self)
 		operation = AsyncOperation(
 			self._readlines, lock=self._lock, callback=callback, data=data)
 		operation.start()
 		return operation
 
 	def _readlines(self):
-		if not self.exists():
-			return []
-		else:
+		try:
 			file = self.open('r')
 			content = file.readlines()
 			self._checkoverwrite(content)
 			return map(_convert_newlines, content)
+		except IOError:
+			raise FileNotFoundError(self)
 
 	def write(self, text):
 		'''Overwrite file with text'''
