@@ -34,10 +34,84 @@ KEYVALS_SLASH = (
 	gtk.gdk.unicode_to_keyval(ord('/')), gtk.gdk.keyval_from_name('KP_Divide'))
 
 
+# UI Environment config. Would properly belong in zim.gui.__init__
+# but defined here to avoid unnecessary dependencies on zim.gui
+ui_environment = {
+	'platform': None, # platform name to trigger platform specific optimizations
+	'maxscreensize': None, # max screensize _if_ fixed by the platform
+	'smallscreen': False, # trigger optimizations for small screens
+}
+
+
+# Check for Maemo environment
+try:
+	import hildon
+	Window = hildon.Window
+	ui_environment['platform'] = 'maemo'
+	if hasattr(Window,'set_app_menu'):
+		ui_environment['maemo_version'] = 'maemo5'
+	else:
+		ui_environment['maemo_version'] = 'maemo4'
+	ui_environment['maxscreensize'] = (800, 480)
+	ui_environment['smallscreen'] = True
+
+	# Maemo gtk UI bugfix: expander-size is set to 0 by default
+	gtk.rc_parse_string('''\
+style "toolkit"
+{
+	GtkTreeView::expander-size = 12
+}
+
+class "GtkTreeView" style "toolkit"
+''' )
+except ImportError:
+	Window = gtk.Window
+
+
 def _encode_xml(text):
 	return text.replace('>', '&gt;').replace('<', '&lt;')
 
 
+def gtk_window_set_default_icon():
+	from zim.config import ZIM_DATA_DIR, XDG_DATA_HOME, XDG_DATA_DIRS
+	iconlist = []
+	if ZIM_DATA_DIR:
+		dir = ZIM_DATA_DIR + '../icons'
+		for name in ('zim16.png', 'zim32.png', 'zim48.png'):
+			file = dir.file(name)
+			if file.exists():
+				pixbuf = gtk.gdk.pixbuf_new_from_file(file.path)
+				iconlist.append(pixbuf)
+	else:
+		sizes = ['16x16', '32x32', '48x48']
+		for dir in [XDG_DATA_HOME] + XDG_DATA_DIRS:
+			for size in sizes:
+				file = dir.file('icons/hicolor/%s/apps/zim.png' % size)
+				if file.exists():
+					sizes.remove(size)
+					pixbuf = gtk.gdk.pixbuf_new_from_file(file.path)
+					iconlist.append(pixbuf)
+			if not sizes:
+				break
+
+	if not iconlist:
+		# fall back to data/zim.png
+		file = zim.config.data_file('zim.png')
+		pixbuf = gtk.gdk.pixbuf_new_from_file(file.path)
+		iconlist.append(pixbuf)
+
+		# also register it as stock since theme apparently is not found
+		factory = gtk.IconFactory()
+		factory.add_default()
+		set = gtk.IconSet(pixbuf=pixbuf)
+		factory.add('zim', set)
+
+
+	if len(iconlist) < 3:
+		logger.warn('Could not find all icon sizes for the application icon')
+	gtk.window_set_default_icon_list(*iconlist)
+
+
 def scrolled_text_view(text=None, monospace=False):
 	'''Initializes a gtk.TextView with sane defaults for displaying a
 	piece of multiline text, wraps it in a scrolled window and returns
@@ -278,14 +352,23 @@ widget "*.zim-statusbar-menubutton" style "zim-statusbar-menubutton-style"
 			self.label = gtk.Label()
 			self.label.set_markup_with_mnemonic(label)
 		else:
-			assert isinstance(label, gtk.Widget)
+			assert isinstance(label, gtk.Label)
 			self.label = label
+
 		self.menu = menu
 		self.button = gtk.ToggleButton()
 		if status_bar_style:
 			self.button.set_name('zim-statusbar-menubutton')
 			self.button.set_relief(gtk.RELIEF_NONE)
-		self.button.add(self.label)
+			widget = self.label
+		else:
+			arrow = gtk.Arrow(gtk.ARROW_UP, gtk.SHADOW_NONE)
+			widget = gtk.HBox(spacing=3)
+			widget.pack_start(self.label, False)
+			widget.pack_start(arrow, False)
+
+
+		self.button.add(widget)
 		# We need to wrap stuff in an eventbox in order to get the gdk.Window
 		# which we need to get coordinates when positioning the menu
 		self.eventbox = gtk.EventBox()
@@ -335,12 +418,17 @@ widget "*.zim-statusbar-menubutton" style "zim-statusbar-menubutton-style"
 		self.button.set_active(False)
 		self.button.handler_unblock(self._clicked_signal)
 
+
 # Need to register classes defining / overriding gobject signals
 gobject.type_register(MenuButton)
 
 
 class InputEntry(gtk.Entry):
-	'''Sub-class of gtk.Entry with support for highlighting errors'''
+	'''Sub-class of gtk.Entry with support for highlighting errors.
+	Also does utf-8 decoding on the proper moments.
+	'''
+
+	# TODO add default hook to set validation function
 
 	style = gtk_get_style()
 	NORMAL_COLOR = style.base[gtk.STATE_NORMAL]
@@ -350,6 +438,15 @@ class InputEntry(gtk.Entry):
 		gtk.Entry.__init__(self)
 		self.input_valid = True
 
+	def get_text(self):
+		'''Like gtk.Entry.get_text() but with utf-8 decoding and
+		trailing whitespace stripped.
+		'''
+		text = gtk.Entry.get_text(self)
+		if not text is None:
+			text = text.decode('utf-8').rstrip()
+		return text
+
 	def get_input_valid(self):
 		return self.input_valid
 
@@ -376,6 +473,7 @@ class PageEntry(InputEntry):
 		self.notebook = notebook
 		self.path_context = path_context
 		self.force_child = False
+		self.force_existing = False
 		self._completing = ''
 
 		self.completion_model = gtk.ListStore(str)
@@ -394,8 +492,14 @@ class PageEntry(InputEntry):
 		self.set_text(':'+path.name)
 
 	def get_path(self):
+		'''Returns a valid Path object or None. If None is returned
+		the widget is flagged as invalid. So e.g. in a dialog you can
+		get a path and refuse to close a dialog if the path is None and
+		the user will automatically be alerted to the missing input.
+		'''
 		name = self.get_text().decode('utf-8').strip()
 		if not name:
+			self.set_input_valid(False)
 			return None
 		elif self.allow_select_root and name == ':':
 			return Path(':')
@@ -408,8 +512,13 @@ class PageEntry(InputEntry):
 				else:
 					path = Path(name)
 			except PageNameError:
+				self.set_input_valid(False)
 				return None
 			else:
+				if self.force_existing:
+					page = self.notebook.get_page(path)
+					if not (page and page.exists()):
+						return None
 				return path
 
 	def clear(self):
@@ -442,6 +551,10 @@ class PageEntry(InputEntry):
 		else:
 			self.set_input_valid(True)
 
+		if self.force_existing:
+			path = self.get_path()
+			self.set_input_valid(not path is None)
+
 		if not self.notebook:
 			return # no completion without a notebook
 
@@ -604,23 +717,23 @@ class Dialog(gtk.Dialog):
 			title=format_title(title),
 			flags=gtk.DIALOG_NO_SEPARATOR|gtk.DIALOG_DESTROY_WITH_PARENT,
 		)
-		self.set_border_width(10)
-		self.vbox.set_spacing(5)
+		if not ui_environment['smallscreen']:
+			self.set_border_width(10)
+			self.vbox.set_spacing(5)
 
 		if hasattr(self, 'uistate'):
 			self.uistate.setdefault('windowsize', defaultwindowsize, check=self.uistate.is_coord)
 		elif hasattr(ui, 'uistate') \
 		and isinstance(ui.uistate, zim.config.ConfigDict):
-			assert isinstance(defaultwindowsize, tuple)
 			key = self.__class__.__name__
 			self.uistate = ui.uistate[key]
 			self.uistate.setdefault('windowsize', defaultwindowsize, check=self.uistate.is_coord)
-			#~ print '>>', self.uistate
 		else:
 			self.uistate = { # used in tests/debug
-				'windowsize': (-1, -1)
+				'windowsize': defaultwindowsize
 			}
 
+		#~ print '>>', self.uistate
 		w, h = self.uistate['windowsize']
 		self.set_default_size(w, h)
 
@@ -696,6 +809,7 @@ class Dialog(gtk.Dialog):
 		same dialog.
 		'''
 		# FIXME FIXME FIXME - this code needs to go in a special class for constructing forms
+		# when doing so define all input types as constants
 		if table is None:
 			table = gtk.Table()
 			table.set_border_width(5)
@@ -704,6 +818,20 @@ class Dialog(gtk.Dialog):
 			self.vbox.add(table)
 		i = table.get_property('n-rows')
 
+		def _hook_label_to_widget(label, widget):
+			# Hook label to follow state of entry widget
+			def _sync_state(widget, spec):
+				label.set_sensitive(widget.get_property('sensitive'))
+				label.set_no_show_all(widget.get_no_show_all())
+				if widget.get_property('visible'):
+					label.show()
+				else:
+					label.hide()
+
+
+			for property in ('visible', 'no-show-all', 'sensitive'):
+				widget.connect_after('notify::%s' % property, _sync_state)
+
 		for field in fields:
 			name, type, label, value = field
 			if type == 'bool':
@@ -720,6 +848,7 @@ class Dialog(gtk.Dialog):
 				button.set_value(v)
 				button.set_range(min, max)
 				button.set_increments(1,5)
+				_hook_label_to_widget(label, button)
 				self.inputs[name] = button
 				table.attach(button, 1,2, i,i+1)
 			elif type == 'list':
@@ -757,6 +886,8 @@ class Dialog(gtk.Dialog):
 						else:
 							assert isinstance(value, Path)
 							entry.set_path(value)
+
+					_hook_label_to_widget(label, entry)
 					self.inputs[name] = entry
 					table.attach(entry, 1,2, i,i+1)
 
@@ -771,6 +902,8 @@ class Dialog(gtk.Dialog):
 					entry.zim_type = type
 					if not value is None:
 						entry.set_text(str(value))
+
+					_hook_label_to_widget(label, entry)
 					self.inputs[name] = entry
 					table.attach(entry, 1,2, i,i+1)
 
@@ -808,7 +941,11 @@ class Dialog(gtk.Dialog):
 		if type == 'dir':
 			dialog = SelectFolderDialog(self)
 		else:
-			dialog = SelectFileDialog(self)
+			if type == 'output-file':
+				action = gtk.FILE_CHOOSER_ACTION_SAVE
+			else:
+				action = gtk.FILE_CHOOSER_ACTION_OPEN
+			dialog = SelectFileDialog(self, action=action)
 			if type == 'image':
 				dialog.add_filter_images()
 		file = dialog.run()
@@ -878,9 +1015,10 @@ class Dialog(gtk.Dialog):
 		else:
 			destroy = True
 
-		w, h = self.get_size()
-		self.uistate['windowsize'] = (w, h)
-		self.save_uistate()
+		if ui_environment['platform'] != 'maemo':
+			w, h = self.get_size()
+			self.uistate['windowsize'] = (w, h)
+			self.save_uistate()
 
 		if destroy:
 			self.destroy()
@@ -1044,11 +1182,15 @@ class FileDialog(Dialog):
 			elif action == gtk.FILE_CHOOSER_ACTION_SAVE:
 				button = (None, gtk.STOCK_SAVE)
 			# else Ok will do
-		Dialog.__init__(self, ui, title,
+
+		if ui_environment['platform'] == 'maemo':
+			defaultsize = (800, 480)
+		else:
+			defaultsize = (500, 400)
+
+		Dialog.__init__(self, ui, title, defaultwindowsize=defaultsize,
 			buttons=buttons, button=button, help_text=help_text, help=help)
-		if self.uistate['windowsize'] == (-1, -1):
-			self.uistate['windowsize'] = (500, 400)
-			self.set_default_size(500, 400)
+
 		self.filechooser = gtk.FileChooserWidget(action=action)
 		self.filechooser.set_do_overwrite_confirmation(True)
 		self.filechooser.set_select_multiple(multiple)
@@ -1123,9 +1265,10 @@ class FileDialog(Dialog):
 
 class SelectFileDialog(FileDialog):
 
-	def __init__(self, ui, title=_('Select File')):
+	def __init__(self, ui, title=_('Select File'), action=gtk.FILE_CHOOSER_ACTION_OPEN):
 		# T: Title of file selection dialog
 		FileDialog.__init__(self, ui, title)
+		self.filechooser.set_action(action)
 		self.file = None
 
 	def do_response_ok(self):
@@ -1242,10 +1385,254 @@ class ProgressBarDialog(gtk.Dialog):
 	#def do_destroy(self):
 	#	logger.debug('Closed ProgressBarDialog')
 
+
 # Need to register classes defining gobject signals
 gobject.type_register(ProgressBarDialog)
 
 
+class Assistant(Dialog):
+	'''Dialog with multi-page input, sometimes also revert to as a
+	"wizard". Similar to gtk.Assistent but does not derive from that
+	class for lack of flexibility in setting the dialog layout.
+
+	Each "page" in the assistant is a step in the work flow. Pages
+	should inherit from the AssistantPage class. Pages share the
+	'uistate' dict with assistant object, and can also use this to
+	communicate state to another page. So each step can change it's
+	look based on state set in the previous step.
+
+	Sub-classes can freely manipulate the flow of pages e.g. by
+	overloading the previous_page() and next_page() methods.
+	'''
+
+	def __init__(self, ui, title, **options):
+		Dialog.__init__(self, ui, title, **options)
+		self.set_border_width(5)
+		self._pages = []
+		self._page = -1
+		self._uistate = self.uistate
+		self.uistate = self._uistate.copy()
+		# Use temporary state, so we can cancel the wizard
+
+		buttons = [b for b in self.action_area.get_children()
+			if not self.action_area.child_get_property(b, 'secondary')]
+		#~ print [b.get_label() for b in buttons]
+		self.ok_button = buttons[0] # HACK: not sure this order fixed
+		self.ok_button.set_no_show_all(True)
+
+		self.back_button = gtk.Button(stock=gtk.STOCK_GO_BACK)
+		self.back_button.connect_object('clicked', self.__class__.previous_page, self)
+		self.action_area.add(self.back_button)
+
+		self.forw_button = gtk.Button(stock=gtk.STOCK_GO_FORWARD)
+		self.forw_button.set_no_show_all(True)
+		self.forw_button.connect_object('clicked', self.__class__.next_page, self)
+		self.action_area.add(self.forw_button)
+
+		self.action_area.reorder_child(self.ok_button, -1)
+
+	def append_page(self, page):
+		'''Append a page'''
+		assert isinstance(page, AssistantPage)
+		self._pages.append(page)
+
+	def run(self):
+		assert self._pages
+		self.set_page(0)
+		Dialog.run(self)
+
+	def get_pages(self):
+		'''Returns a list with AssistantPage objects'''
+		return self._pages
+
+	def set_page(self, i):
+		'''Go to page i in the assistant'''
+		if i < 0 or i >= len(self._pages):
+			return False
+
+		# Wrap up previous page
+		if self._page > -1:
+			try: # hack needed for filechooser valid in gtk < 2.12
+				self._pages[self._page]._check_valid()
+			except Exception, error:
+				ErrorDialog(self, error).run()
+				return False
+			else:
+				self._pages[self._page].save_uistate()
+
+		# Remove previous page
+		for child in self.vbox.get_children():
+			if not isinstance(child, gtk.ButtonBox):
+				self.vbox.remove(child)
+
+		self._page = i
+		page = self._pages[self._page]
+
+		# Add page title - use same color as used by gtkassistent.c
+		ebox = gtk.EventBox()
+		style = gtk_get_style()
+		ebox.modify_fg(gtk.STATE_NORMAL, style.fg[gtk.STATE_SELECTED])
+		ebox.modify_bg(gtk.STATE_NORMAL, style.bg[gtk.STATE_SELECTED])
+
+		hbox = gtk.HBox()
+		hbox.set_border_width(5)
+		ebox.add(hbox)
+		self.vbox.pack_start(ebox, False)
+
+		label = gtk.Label()
+		label.set_markup('<b>' + page.title + '</b>')
+		hbox.pack_start(label, False)
+		label = gtk.Label()
+		label.set_markup('<b>(%i/%i)</b>' % (self._page+1, len(self._pages)))
+		hbox.pack_end(label, False)
+
+		# Add actual page
+		self.vbox.add(page)
+		self.vbox.show_all()
+		page.init_uistate()
+
+		self.back_button.set_sensitive(self._page > 0)
+		if self._page < len(self._pages) - 1:
+			self.forw_button.show()
+			self.ok_button.hide()
+		else:
+			self.forw_button.hide()
+			self.ok_button.show()
+
+		self._update_valid()
+		page.connect('input-valid-changed', self._update_valid)
+
+		return True
+
+	def _update_valid(self, *a):
+		page = self._pages[self._page]
+		ok = page.get_input_valid()
+		self.forw_button.set_sensitive(ok)
+		self.ok_button.set_sensitive(ok)
+
+	def next_page(self):
+		'''Go forward to the next page'''
+		return self.set_page(self._page + 1)
+
+	def previous_page(self):
+		'''Go back to the previous page'''
+		return self.set_page(self._page - 1)
+
+	def do_response(self, id):
+		# Wrap up previous page
+		if self._page > -1:
+			try: # hack needed for filechooser valid in gtk < 2.12
+				self._pages[self._page]._check_valid()
+			except Exception, error:
+				ErrorDialog(self, error).run()
+				return False
+			else:
+				self._pages[self._page].save_uistate()
+
+		if id == gtk.RESPONSE_OK:
+			self._uistate.update(self.uistate)
+		Dialog.do_response(self, id)
+
+
+class AssistantPage(gtk.VBox):
+	'''Base class for pages in an Assistant dialog. Should have an
+	attribute 'title'. Also will have an attribute 'uistate' which
+	is set by the constructor to link to the dialog. This uistate is
+	shared between all pages in the same dialog.
+
+	The input needs to be valid before the user is allowed to continue.
+	You can set valid state directly or use the convenience functions
+	to hook widgets that need to be valid.
+	'''
+
+	# define signals we want to use - (closure type, return type and arg types)
+	__gsignals__ = {
+		'input-valid-changed': (gobject.SIGNAL_RUN_LAST, None, ()),
+	}
+
+	title = ''
+
+	def __init__(self, assistant):
+		gtk.VBox.__init__(self)
+		self.set_border_width(5)
+		self.uistate = assistant.uistate
+		self._input_valid = True
+		self._input_valid_widgets = []
+
+	def init_uistate(self):
+		'''Hook called when a new page is shown in the dialog. Should
+		be used to update uistate according to input of other pages.
+		'''
+		pass
+
+	def save_uistate(self):
+		'''Hook called when leaving the current page. Should set uistate
+		to reflect user input. Should not fail on validation.
+		'''
+		pass
+
+	def set_input_valid(self, valid):
+		self._input_valid = valid
+		self.emit('input-valid-changed')
+
+	def get_input_valid(self):
+		return self._input_valid
+
+	def add_validation_widget(self, widget):
+		'''Hook an InputEntry or gtk.FileChooese object for input
+		validation. If the widget is visible and sensitive it needs
+		to be valid in order for the page to be valid.
+
+		For FileChooser objects being valid means a file or folder
+		was selected.
+		'''
+		if isinstance(widget, gtk.FileChooser):
+			if gtk.gtk_version >= (2, 12, 0):
+				widget.connect_after('file-set', self._update_valid)
+		else:
+			widget.connect_after('changed', self._update_valid)
+
+		widget.connect_after('notify::sensitive', self._update_valid)
+		widget.connect_after('notify::visible', self._update_valid)
+
+		self._input_valid_widgets.append(widget)
+		self._update_valid()
+
+	def _update_valid(self, *a):
+		valid = True
+		for widget in self._input_valid_widgets:
+			if not widget.get_property('sensitive') \
+			or not widget.get_property('visible'):
+				continue
+
+			if isinstance(widget, gtk.FileChooser):
+				if gtk.gtk_version >= (2, 12, 0):
+					valid = not widget.get_filename() is None
+				else:
+					pass
+			else:
+				valid = widget.get_input_valid()
+
+		self.set_input_valid(valid)
+
+	def _check_valid(self):
+		# HACK: Called in a try .. except block before finalizing page
+		# needed because missing signal for filechooserbutton in gtk < 2.12
+		# We can remove it when we get rid of using filechooserbuttons
+		if self._input_valid_widgets:
+			for widget in self._input_valid_widgets:
+				if isinstance(widget, gtk.FileChooser) \
+				and widget.get_property('sensitive') \
+				and widget.get_property('visible'):
+					if widget.get_filename() is None:
+						raise AssertionError, 'Missing file name'
+		else:
+			return self.get_input_valid()
+
+# Need to register classes defining gobject signals
+gobject.type_register(AssistantPage)
+
+
 class ImageView(gtk.Layout):
 
 	SCALE_FIT = 1 # scale image with the window (if it is bigger)
