@@ -53,7 +53,7 @@ about the actual existence of the page etc.
 
 If a non-exising page is requested the store should check if we are allowed
 to create the page. If so, a new page object should be returned, but actually
-creating the page can be delayed untill content is stored in it. Creating
+creating the page can be delayed until content is stored in it. Creating
 the page also implicitly creates all of it's parents page, since it should
 be visible in the hierarchy of page listings. If we are not allowed to create
 the page (e.g. in case of a read-only notebook) no page object should be
@@ -66,7 +66,10 @@ return an empty list.
 from __future__ import with_statement
 
 import sys
+import re
+import codecs
 
+import zim.fs
 from zim.fs import *
 from zim.parsing import is_url_re
 
@@ -80,15 +83,45 @@ def get_store(name):
 	return mod
 
 
+def _url_encode_on_error(error):
+	string = error.object
+	section = string[error.start:error.end].encode('utf-8')
+	replace = u''
+	for char in section:
+		replace += u'%%%02X' % ord(char)
+	return replace, error.end
+
+codecs.register_error('urlencode', _url_encode_on_error)
+
+
 def encode_filename(pagename):
 	'''Encodes a pagename to a filename. Namespaces are mapped to directories.
 	Returns basename without extension.
+	Characters not allowed for the filesystem are encoded with url encoding.
 	'''
+	assert not '%' in pagename # just to be sure
+	if not zim.fs.ENCODING in ('utf-8', 'mbcs'):
+		# if not utf-8 we may not be able to encode all characters
+		# enforce safe encoding, but do not actually encode here
+		# ('mbcs' means we are running on windows and filesystem can
+		# handle unicode natively )
+		pagename = pagename.encode(zim.fs.ENCODING, 'urlencode')
+		pagename = pagename.decode(zim.fs.ENCODING)
 	return pagename.replace(':', '/').replace(' ', '_')
 
 
+_url_decode_re = re.compile('%([a-fA-F0-9]{2})')
+
+def _url_decode(match):
+	return chr(int(match.group(1), 16))
+
+
 def decode_filename(filename):
 	'''Decodes a file basename to a pagename'''
+	if zim.fs.ENCODING != 'utf-8':
+		filename = filename.encode('utf-8')
+		filename = _url_decode_re.sub(_url_decode, filename)
+		filename = filename.decode('utf-8')
 	return filename.replace('/', ':').replace('_', ' ')
 
 
